unit musicd0; { General }

interface

uses
  StdCtrls, ExtCtrls,
  SysUtils, WinProcs, Classes, Controls, Graphics, Forms, Dialogs,
  IniFiles;

const
  { global constants : general }
  GC_ProgIdn       = 'MusicD';
  GC_ProgVers      = 'v1.1';
  GC_ProgTarget    = 'Windows 95/NT';
  GC_Copyright     = 'Copyright  2000, J.R. Ferguson';
  GC_Email         = 'j.r.ferguson@iname.com';
  GC_Website       = 'http://hello.to/ferguson';
  GC_LinkEmail     = true;
  GC_LinkWebsite   = true;
  GC_SettingExt    = '.ini';
  GC_MinWidth      = 270;
  GC_MinHeight     = 210;

  GC_MinKeyCount   =  3;
  GC_DflKeyCount   =  8;
  GC_MaxKeyCount   = 10;
  GC_DflFixedStart = true;
  GC_DflPlayTunes  = true;
  GC_DflBlackKeys  = true;

  GC_MinSpeed      =  1;
  GC_DflSpeed      = 12;
  GC_MaxSpeed      = 24;
  GC_TimeFactor    = 10;

type
  T_Language       = (lng_US,lng_NL);
  T_TextIndex      = (
    S_ProgDate,
    S_ProgTitle,
    S_MnuGame,
    S_MnuGameStart,
    S_MnuGameExit,
    S_MnuOptions,
    S_MnuOptToolBar,
    S_MnuOptHints,
    S_MnuOptFont,
    S_MnuOptSetting,
    S_MnuOptLanguage,
    S_MnuHelp,
    S_MnuHelpRules,
    S_MnuHelpAbout,
    S_HintExit,
    S_HintHints,
    S_HintFont,
    S_HintSettings,
    S_HintRules,
    S_BtnStart,
    S_StatusNotes,
    S_StatusScore,
    S_StatusBestScore,
    S_SettingDlgTitle,
    S_SettingKeyCount,
    S_SettingFixedStart,
    S_SettingPlayTunes,
    S_SettingBlackKeys,
    S_SettingSpeed,
    S_SettingSlow,
    S_SettingFast,
    S_SettingDefault,
    S_SettingCancel,
    S_SettingOK,
    S_AboutBoxTitle,
    S_InitialPrompt,
    S_PlayPrompt,
    S_RepeatPrompt,
    S_GameOver,
    S_About,
    S_ErrEmail,
    S_ErrURL,
    S_Rules);

  T_Tone           = 1..GC_MaxKeyCount;
  T_Duration       = Word;
  T_Speed          = GC_MinSpeed..GC_MaxSpeed;
  T_KeyArray       = array[T_Tone] of TButton;
  T_BlackKeyArray  = array[T_Tone] of TPanel;
  T_GameSettings   = record
    KeyCount       : T_Tone;
    FixedStart     : boolean;
    PlayTunes      : boolean;
    BlackKeys      : boolean;
    Speed          : T_Speed;
  end;
  T_GameState      = (gs_Idle,
                      gs_Play,
                      gs_Repeat);

const
  Hz               : array[T_Tone] of Word =
    (523,587,659,698,784,880,988,1047,1175,1319);
  GC_DflSettings   : T_GameSettings = (
    KeyCount       : GC_DflKeyCount;
    FixedStart     : GC_DflFixedStart;
    PlayTunes      : GC_DflPlayTunes;
    BlackKeys      : GC_DflBlackKeys;
    Speed          : GC_DflSpeed);
  GC_ControlsOK    : boolean = false;

  GC_Text: array[T_TextIndex,T_Language] of string = (
  { S_ProgDate         } ('Mar 12, 2000'        ,'12 maart 2000'),
  { S_ProgTitle        } ('Music Memory'        ,'Muzikaal geheugenspel'),
  { S_MnuGame          } ('&Game'               ,'&Spel'),
  { S_MnuGameStart     } ('&Start'              ,'&Begin spel'),
  { S_MnuGameExit      } ('E&xit'               ,'E&xit'),
  { S_MnuOptions       } ('&Options'            ,'&Opties'),
  { S_MnuOptToolBar    } ('&Toolbar'            ,'&Knoppenbalk'),
  { S_MnuOptHints      } ('&Hints'              ,'&Hints'),
  { S_MnuOptFont       } ('&Font'               ,'&Font'),
  { S_MnuOptSetting    } ('Game &settings...'   ,'&Instelling...'),
  { S_MnuOptLanguage   } ('&Language'           ,'&Taal'),
  { S_MnuHelp          } ('&Help'               ,'&Help'),
  { S_MnuHelpRules     } ('&Rules'              ,'&Spelregels'),
  { S_MnuHelpAbout     } ('&About'              ,'&Info'),
  { S_HintExit         } ('Exit'                ,'Exit'),
  { S_HintHints        } ('Hints'               ,'Hints'),
  { S_HintFont         } ('Font'                ,'Font'),
  { S_HintSettings     } ('Settings'            ,'Instelling'),
  { S_HintRules        } ('Rules'               ,'Spelregels'),
  { S_BtnStart         } ('&Start'              ,'&Begin spel'),
  { S_StatusNotes      } ('notes'               ,'noten'),
  { S_StatusScore      } ('Score:'              ,'Score:'),
  { S_StatusBestScore  } ('Best score:'         ,'Beste score:'),
  { S_SettingDlgTitle  } ('Game settings'       ,'Instellingen'),
  { S_SettingKeyCount  } ('Number of &keys'     ,'&Aantal toetsen'),
  { S_SettingFixedStart} ('&Fixed melody start' ,'&Vaste beginmelodie'),
  { S_SettingPlayTunes } ('Play &tunes'         ,'&Herkenningsmelodie'),
  { S_SettingBlackKeys } ('Show &black keys'    ,'Toon &zwarte toetsen'),
  { S_SettingSpeed     } ('&Speed'              ,'&Snelheid'),
  { S_SettingSlow      } ('slow'                ,'langzaam'),
  { S_SettingFast      } ('fast'                ,'snel'),
  { S_SettingDefault   } ('&Default'            ,'&Beginwaarden'),
  { S_SettingCancel    } ('Cancel'              ,'Annuleren'),
  { S_SettingOK        } ('OK'                  ,'OK'),
  { S_AboutBoxTitle    } ('About'               ,'Info'),
  { S_InitialPrompt    } ('Press Start to begin','Kies "Begin spel"'),
  { S_PlayPrompt       } ('Listen carefully ...','Luister goed ...'),
  { S_RepeatPrompt     } ('Your turn ...'       ,'Speel het na ...'),
  { S_GameOver         } ('Game over'           ,'Einde spel'),
  { S_About            } ('About '              ,'Over '),
  { S_ErrEmail         } ('failed to create e-mail',
                          'kan e-mail programma niet starten'),
  { S_ErrURL           } ('failed to launch web browser',
                          'kan web-browser niet starten'),
  { S_Rules            } (
    'Chose "Start". Listen well, then play the same melody, '+
    'using the mouse or the numerical keys.',
    'Kies "Begin spel". Luister goed en speel de melodie na '+
    'met de muis of met de cijfertoetsen.')
  );

var
  GV_SettingFnm    : String;
  GV_GameSettings  : T_GameSettings;
  GV_Language      : T_Language;

function  LS(V_TextIndex: T_TextIndex): String;
function  ToneToChar(V_Tone: T_Tone): char;
function  CharToTone(V_Char: char): T_Tone;
function  DuraToChar(V_Duration: T_Duration): char;
function  CharToDura(V_Char: char): T_Duration;
procedure SetCursor(V_NewCursor: TCursor);
procedure RestoreCursor;
procedure ReadOptions;
function  WriteOptions: boolean;

implementation

Uses musicd1;

var
  OldCursor: TCursor;

function  LS(V_TextIndex: T_TextIndex): String;
begin Result:= GC_Text[V_TextIndex,GV_Language]; end;

function  ToneToChar(V_Tone: T_Tone): char;
begin Result:= chr(ord('0')+V_Tone); end;

function  CharToTone(V_Char: char): T_Tone;
begin Result:= ord(V_Char) - ord('0'); end;

function  DuraToChar(V_Duration: T_Duration): char;
begin Result:= chr(ord('0')+V_Duration); end;

function  CharToDura(V_Char: char): T_Duration;
begin Result:= ord(V_Char) - ord('0'); end;

procedure SetCursor(V_NewCursor: TCursor);
begin OldCursor:= Screen.Cursor; Screen.Cursor:= V_NewCursor; end;

procedure RestoreCursor;
begin Screen.Cursor:= OldCursor; end;

procedure ReadOptions;
var Bold, Italic: boolean; IniFile: TIniFile;
begin
  SetCursor(crHourglass); IniFile:= TIniFile.Create(GV_SettingFnm);
  if IniFile <> nil then with IniFile, MainWindow do try
    Left   := ReadInteger('Position','Left'  ,Left);
    Top    := ReadInteger('Position','Top'   ,Top);
    Width  := ReadInteger('Position','Width' ,Width);
    Height := ReadInteger('Position','Height',Height);
    with FontDialog.Font do begin
      Name    := ReadString ('Font' ,'Name'  ,Name);
      Size    := ReadInteger('Font' ,'Size'  ,Size);
      Color   := ReadInteger('Font' ,'Color' ,Color);
      Bold    := ReadBool   ('Font' ,'Bold'  ,fsBold   in Style);
      Italic  := ReadBool   ('Font' ,'Italic',fsItalic in Style);
      Style   := [];
      if Bold   then Style := Style + [fsBold];
      if Italic then Style := Style + [fsItalic];
      ApplyFont;
    end;
    with Toolbar    do Visible := ReadBool('Options','Toolbar' ,Visible);
    with MainWindow do ShowHint:= ReadBool('Options','Hints'   ,ShowHint);
    with GV_GameSettings do begin
      KeyCount   := ReadInteger('Options','KeyCount'  ,KeyCount  );
      FixedStart := ReadBool   ('Options','FixedStart',FixedStart);
      PlayTunes  := ReadBool   ('Options','PlayTunes' ,PlayTunes );
      BlackKeys  := ReadBool   ('Options','BlackKeys' ,BlackKeys );
      Speed      := ReadInteger('Options','Speed'     ,Speed     );
    end;
    GV_Language  := T_Language(
                    ReadInteger('Options','Language'  ,ord(GV_Language)));
  finally IniFile.Free; AdjustControls; RestoreCursor; end;
end;

function  WriteOptions: boolean;
var IniFile: TIniFile;
begin
  Result:= true;
  SetCursor(crHourglass); IniFile:= TIniFile.Create(GV_SettingFnm);
  if IniFile <> nil then with IniFile, MainWindow do try try
    WriteInteger  ('Position','Left'  ,Left);
    WriteInteger  ('Position','Top'   ,Top);
    WriteInteger  ('Position','Width' ,Width);
    WriteInteger  ('Position','Height',Height);
    with FontDialog.Font do begin
      WriteString ('Font','Name'  ,Name);
      WriteInteger('Font','Size'  ,Size);
      WriteInteger('Font','Color' ,Color);
      WriteBool   ('Font','Bold'  ,fsBold   in Style);
      WriteBool   ('Font','Italic',fsItalic in Style);
    end;
    WriteBool('Options','Toolbar' ,Toolbar.Visible);
    WriteBool('Options','Hints'   ,MainWindow.ShowHint);
    with GV_GameSettings do begin
      WriteInteger('Options','KeyCount'  ,KeyCount  );
      WriteBool   ('Options','FixedStart',FixedStart);
      WriteBool   ('Options','PlayTunes' ,PlayTunes );
      WriteBool   ('Options','BlackKeys' ,BlackKeys );
      WriteInteger('Options','Speed'     ,Speed     );
    end;
    WriteInteger  ('Options','Language'  ,ord(GV_Language));
  finally IniFile.Free; RestoreCursor; end;
  except on E: Exception do
    Result:= MessageDlg(E.Message,mtError,mbOKCancel,0) = mrOk;
  end;
end;

end.

