{ Main Window }
unit musicd1;

interface

uses
  musicd0,
  SndLib,
  ExtCtrls, Menus, Dialogs, StdCtrls, Controls, Buttons, Classes,
  Windows, Messages, SysUtils, Graphics, Forms, ComCtrls;

const
  st_Notes    = 0;
  st_Score    = 3;
  st_BestScore= 5;

type
  TMainWindow = class(TForm)
    ToolBar: TPanel;
    FontDialog: TFontDialog;
    MainMenu: TMainMenu;
    MnuGame: TMenuItem;
    MnuGameStart: TMenuItem;
    MnuGameExit: TMenuItem;
    MnuOptions: TMenuItem;
    MnuOptToolBar: TMenuItem;
    MnuOptHints: TMenuItem;
    MnuOptFont: TMenuItem;
    MnuOptSetting: TMenuItem;
    MnuOptLanguage: TMenuItem;
    MnuOptLangUS: TMenuItem;
    MnuOptLangNL: TMenuItem;
    MnuHelp: TMenuItem;
    MnuHelpRules: TMenuItem;
    MnuHelpAbout: TMenuItem;
    BtnExit: TSpeedButton;
    BtnHints: TSpeedButton;
    BtnFont: TSpeedButton;
    BtnHelp: TSpeedButton;
    PnlPrompt: TPanel;
    PnlStart: TPanel;
    PnlKeys: TPanel;
    BtnStart: TButton;
    Key01: TButton;
    Key02: TButton;
    Key03: TButton;
    Key04: TButton;
    Key05: TButton;
    Key06: TButton;
    Key07: TButton;
    Key08: TButton;
    Key09: TButton;
    Key10: TButton;
    BtnSetting: TSpeedButton;
    Timer: TTimer;
    StatusBar: TStatusBar;
    BlackKey01: TPanel;
    BlackKey02: TPanel;
    BlackKey04: TPanel;
    BlackKey05: TPanel;
    BlackKey06: TPanel;
    BlackKey08: TPanel;
    BlackKey09: TPanel;
    BtnLangUS: TSpeedButton;
    BtnLangNL: TSpeedButton;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure MnuGameStartClick(Sender: TObject);
    procedure MnuGameExitClick(Sender: TObject);
    procedure MnuOptionsClick(Sender: TObject);
    procedure MnuOptToolBarClick(Sender: TObject);
    procedure MnuOptHintsClick(Sender: TObject);
    procedure MnuOptFontClick(Sender: TObject);
    procedure MnuOptSettingClick(Sender: TObject);
    procedure MnuOptLangUSClick(Sender: TObject);
    procedure MnuOptLangNLClick(Sender: TObject);
    procedure MnuOptLanguageClick(Sender: TObject);
    procedure MnuHelpRulesClick(Sender: TObject);
    procedure MnuHelpAboutClick(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
    procedure KeyClick(Sender: TObject);
    procedure FontDialogApply(Sender: TObject; Wnd: Integer);
    procedure FormResize(Sender: TObject);
  private
    procedure WMGetMinMaxInfo(var V_Msg: Tmessage); message WM_GetMinMaxInfo;
  public
    Key       : T_KeyArray;
    BlackKey  : T_BlackKeyArray;
    KeyQueue  : String;
    Melody    : String;
    GameState : T_GameState;
    ActiveGame: boolean;
    Playing   : boolean;
    Simulating: boolean;
    Closing   : boolean;
    Tone      : T_Tone;
    Duration  : T_Duration;
    Speed     : T_Speed;
    RepeatCnt : integer;
    PlayCnt   : integer;
    Score     : integer;
    BestScore : integer;
    procedure GameReset;
    procedure WrongNote;
    procedure ApplyFont;
    procedure AdjustControls;
    procedure AdjustPanes;
    procedure AdjustStatusBar;
    procedure SetLanguage(V_Language: T_Language);
    function  FindKeyTone(V_Button: TButton; var V_Tone: T_Tone): boolean;
    procedure PressKey(V_Tone: T_Tone);
    procedure AddToMelody(V_Tone: T_Tone; V_Duration: T_Duration);
    function  MelodyLength: integer;
    function  MelodyTone(V_Index: integer): T_Tone;
    function  MelodyDuration(V_Index: integer): T_Duration;
    procedure ToStartTune;
    procedure ToEndTune;
    procedure ToAddNote;
    procedure ToPlayMelody;
    procedure ToPlayNote(V_Tone: T_Tone; V_Duration: T_Duration);
    procedure ToKeyPressed(V_Tone: T_Tone);
    procedure ToWait(V_Duration: T_Duration);
    procedure ToCheckState;
    procedure ToCheckQueue;
  end;

var
  MainWindow: TMainWindow;

implementation

uses musicd2, musicd3;

{$R *.DFM}


{ Private }

procedure TMainWindow.WMGetMinMaxInfo(var V_Msg: Tmessage);
begin with PMinMaxInfo(V_Msg.lparam)^.ptMinTrackSize do begin
  x:= GC_MinWidth;
  y:= GC_MinHeight;
end; end;


{ Public }

procedure   TMainWindow.GameReSet;
begin
  SndSoundOff;
  Playing   := false;
  Melody    := '';
  KeyQueue  := '';
  PlayCnt   := 0;
  Score     := 0;
  Speed     := GV_GameSettings.Speed;
  ActiveGame:= false;
  AdjustStatusBar;
  PnlPrompt.Caption:= LS(S_InitialPrompt);
  GameState := gs_Idle;
end;

procedure   TMainWindow.WrongNote;
begin
  if Score > BestScore then BestScore:= Score;
  AdjustStatusBar;
  ShowMessage(LS(S_GameOver));
  GameReset;
end;

procedure TMainWindow.ApplyFont;
begin Font:= FontDialog.Font; end;

procedure TMainWindow.AdjustControls;
begin
  BtnHints.Down        := ShowHint;
  BtnLangUS.Down       := GV_Language = lng_US;
  BtnLangNL.Down       := GV_Language = lng_NL;
end;

procedure TMainWindow.AdjustPanes;
var i: T_Tone; FieldWidth, Distance: integer;
begin if GC_ControlsOK then begin
  BtnStart.Width:= PnlStart.Width - 2*BtnStart.Left;
  with GV_GameSettings do begin
    for i:= 1 to GC_MaxKeyCount do begin
      Key[i].Hide;
      if BlackKey[i] <> nil then BlackKey[i].Hide;
    end;
    for i:= 1 to KeyCount do begin
      FieldWidth := PnlKeys.Width - Key01.Left;
      Distance   := FieldWidth div KeyCount;
      with Key[i] do begin
        Width  := Distance;
        Height := PnlKeys.Height - 2*Key01.Top;
        Left   := Key01.Left + (i-1)*Distance;
        Show;
      end;
      if BlackKeys and (BlackKey[i]<>nil) then with BlackKey[i] do begin
        Width  := 3*Key[i].Width  div 5;
        Height := 3*Key[i].Height div 5;
        Left   := Key[i].Left + Distance - Width div 2;
        if i=KeyCount then Width:= Width div 2;
        Show;
      end;
    end;
  end;
end end;

procedure TMainWindow.AdjustStatusBar;
  function IntStr(n: integer): String;
  begin if n=0 then Result:= '' else Result:= IntToStr(n); end;
begin with StatusBar.Panels do begin
  Items[st_Notes    ].Text:= IntStr(MelodyLength);
  Items[st_Score    ].Text:= IntStr(Score);
  Items[st_BestScore].Text:= IntStr(BestScore);
end end;

procedure TMainWindow.SetLanguage(V_Language: T_Language);
begin
  GV_Language:= V_Language;
  AdjustControls;
  Application.Title      := LS(S_ProgTitle);
  MainWindow.Caption     := LS(S_ProgTitle);
  BtnStart.Caption       := LS(S_BtnStart);
  MnuGame.Caption        := LS(S_MnuGame);
  MnuGameStart.Caption   := LS(S_MnuGameStart);
  MnuGameExit.Caption    := LS(S_MnuGameExit);
  MnuOptions.Caption     := LS(S_MnuOptions);
  MnuHelp.Caption        := LS(S_MnuHelp);
  MnuOptToolBar.Caption  := LS(S_MnuOptToolBar);
  MnuOptHints.Caption    := LS(S_MnuOptHints);
  MnuOptFont.Caption     := LS(S_MnuOptFont);
  MnuOptSetting.Caption  := LS(S_MnuOptSetting);
  MnuOptLanguage.Caption := LS(S_MnuOptLanguage);
  MnuHelp.Caption        := LS(S_MnuHelp);
  MnuHelpRules.Caption   := LS(S_MnuHelpRules);
  MnuHelpAbout.Caption   := LS(S_MnuHelpAbout);
  BtnExit.Hint           := LS(S_HintExit);
  BtnHints.Hint          := LS(S_HintHints);
  BtnFont.Hint           := LS(S_HintFont);
  BtnSetting.Hint        := LS(S_HintSettings);
  BtnHelp.Hint           := LS(S_HintRules);
  BtnStart.Caption       := LS(S_BtnStart);
  with StatusBar do begin
    Panels[1].Text       := LS(S_StatusNotes);
    Panels[2].Text       := LS(S_StatusScore);
    Panels[4].Text       := LS(S_StatusBestScore);
  end;
  case GameState of
    gs_Idle   : PnlPrompt.Caption:= LS(S_InitialPrompt);
    gs_Play   : PnlPrompt.Caption:= LS(S_PlayPrompt);
    gs_Repeat : PnlPrompt.Caption:= LS(S_RepeatPrompt);
  end;
end;

function TMainWindow.FindKeyTone(V_Button:TButton; var V_Tone:T_Tone):boolean;
var searching: boolean;
begin
  V_Tone:= 1; searching:= Key[V_Tone] <> V_Button;
  while searching and (V_Tone < GC_MaxKeyCount) do begin
    inc(V_Tone);
    searching:= Key[V_Tone] <> V_Button;
  end;
  Result:= not searching;
end;

procedure TMainWindow.PressKey(V_Tone: T_Tone);
begin
  Simulating:= true;
  Key[V_Tone].Click;
end;

procedure TMainWindow.AddToMelody(V_Tone: T_Tone; V_Duration: T_Duration);
begin Melody:= Melody + ToneToChar(V_Tone) + DuraToChar(V_Duration); end;

function  TMainWindow.MelodyLength: integer;
begin Result:= Length(Melody) div 2; end;

function  TMainWindow.MelodyTone(V_Index: integer): T_Tone;
begin Result:= CharToTone(Melody[2*V_Index-1]); end;

function  TMainWindow.MelodyDuration(V_Index: integer): T_Duration;
begin Result:= CharToDura(Melody[2*V_Index]); end;

procedure TMainWindow.ToStartTune;
var i: T_Tone;
begin
  ActiveGame := false;
  Speed      := GC_DflSpeed;
  Melody     := '';
  AddToMelody(1,2);
  for i:= 2 to 7 do AddToMelody(i,1);
  AddToMelody(8,3);
  ToPlayMelody;
end;

procedure   TMainWindow.ToEndTune;
var i: T_Tone;
begin
  ActiveGame := false;
  Speed      := GC_DflSpeed;
  Melody     := '';
  AddToMelody(8,2);
  for i:= 7 downto 2 do AddToMelody(i,1);
  AddToMelody(1,3);
  ToPlayMelody;
end;

procedure   TMainWindow.ToAddNote;
var i,n: integer;
begin with GV_GameSettings do begin
  if FixedStart then AddToMelody(1+Random(KeyCount),3)
  else begin
    n:= MelodyLength + 1;
    Melody:= '';
    for i:= 1 to n do AddToMelody(1+Random(KeyCount),3)
  end;
  PnlPrompt.Caption:= LS(S_PlayPrompt);
  AdjustStatusBar;
  ToPlayMelody;
end; end;

procedure TMainWindow.ToPlayMelody;
begin if MelodyLength > 0 then begin
  PlayCnt   := 0;
  Playing   := true;
  GameState := gs_Play;
  ToWait(2);
end; end;

procedure TMainWindow.ToKeyPressed(V_Tone: T_Tone);
begin
  if Playing then begin
    if GameState in [gs_Idle,gs_Repeat] then
      KeyQueue:= KeyQueue + ToneToChar(V_Tone)
  end
  else begin
    Tone     := V_Tone;
    Duration := 3;
    ToPlayNote(V_Tone,3);
  end;
end;

procedure TMainWindow.ToPlayNote(V_Tone: T_Tone; V_Duration: T_Duration);
begin
  if Key[V_Tone].Visible then Key[V_Tone].SetFocus;
  Playing:= true;
  SndSoundOn(Hz[V_Tone]);
  ToWait(V_Duration);
end;

procedure   TMainWindow.ToWait(V_Duration: T_Duration);
begin
  Timer.Enabled := false;
  Timer.Interval:= (2+GC_MaxSpeed-Speed)*V_Duration*GC_TimeFactor;
  Timer.Enabled := true;
end;

procedure TMainWindow.ToCheckState;
begin case GameState of
  gs_Idle   : ToCheckQueue;
  gs_Play   : if PlayCnt < MelodyLength then begin
                Inc(PlayCnt);
                Duration:= MelodyDuration(PlayCnt);
                PressKey(MelodyTone(PlayCnt));
              end
              else begin
                Playing := false;
                Speed   := GV_GameSettings.Speed;
                PlayCnt := 0;
                if ActiveGame then begin
                  RepeatCnt:= 0;
                  AdjustStatusBar;
                  PnlPrompt.Caption:= LS(S_RepeatPrompt);
                  GameState:= gs_Repeat;
                end
                else if Closing then Close
                else begin
                  GameState:= gs_Idle;
                  Playing  := false;
                end;
              end;
  gs_Repeat : begin
                Inc(RepeatCnt);
                if Tone <> MelodyTone(RepeatCnt) then begin
                  KeyQueue:= '';
                  WrongNote;
                end
                else begin
                  if RepeatCnt = MelodyLength then begin
                    KeyQueue:= '';
                    Inc(Score);
                    AdjustStatusBar;
                    ToAddNote;
                  end
                  else begin
                    AdjustStatusBar;
                    ToCheckQueue;
                  end;
                end;
              end;
end end;

procedure TMainWindow.ToCheckQueue;
begin if KeyQueue <> '' then begin
  Tone    := CharToTone(KeyQueue[1]); Delete(KeyQueue,1,1);
  Duration:= 3;
  PressKey(Tone);
end; end;


{ Generated }

procedure TMainWindow.FormCreate(Sender: TObject);
begin
  Icon:= Application.Icon;
  ApplyFont;
end;

procedure TMainWindow.FormShow(Sender: TObject);
begin
  GV_GameSettings    := GC_DflSettings;
  GV_SettingFnm      := ChangeFileExt(Application.ExeName,GC_SettingExt);
  GV_Language        := lng_US;
  ReadOptions;
  SetLanguage(GV_Language);
  Key[01]   := Key01;
  Key[02]   := Key02;
  Key[03]   := Key03;
  Key[04]   := Key04;
  Key[05]   := Key05;
  Key[06]   := Key06;
  Key[07]   := Key07;
  Key[08]   := Key08;
  Key[09]   := Key09;
  Key[10]   := Key10;
  BlackKey[01] := BlackKey01;
  BlackKey[02] := BlackKey02;
  BlackKey[03] := nil;
  BlackKey[04] := BlackKey04;
  BlackKey[05] := BlackKey05;
  BlackKey[06] := BlackKey06;
  BlackKey[07] := nil;
  BlackKey[08] := BlackKey08;
  BlackKey[09] := BlackKey09;
  BlackKey[10] := nil;
  KeyQueue  := '';
  Playing   := false;
  Closing   := false;
  BestScore := 0;
  AdjustPanes;
  Randomize;
  GameReset;
  GC_ControlsOK:= true;
  if GV_GameSettings.PlayTunes then ToStartTune;
end;

procedure TMainWindow.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  GameReset;
  PnlPrompt.Caption:= '';
  CanClose:= WriteOptions;
end;

procedure TMainWindow.MnuGameExitClick(Sender: TObject);
begin
  if GV_GameSettings.PlayTunes then begin
    Closing:= true;
    ToEndTune;
  end
  else Close;
end;

procedure TMainWindow.MnuOptionsClick(Sender: TObject);
begin
  MnuOptToolbar.Checked  := Toolbar.Visible;
  MnuOptHints.Enabled    := Toolbar.Visible;
  if MnuOptHints.Enabled then
    MnuOptHints.Checked  := ShowHint
  else
    MnuOptHints.Checked  := false;
end;

procedure TMainWindow.MnuOptToolBarClick(Sender: TObject);
begin
  Toolbar.Visible:= not Toolbar.Visible;
end;

procedure TMainWindow.MnuOptHintsClick(Sender: TObject);
begin
  ShowHint:= not ShowHint;
  AdjustControls;
end;

procedure TMainWindow.MnuOptFontClick(Sender: TObject);
begin
  if FontDialog.Execute then ApplyFont;
end;

procedure TMainWindow.MnuOptLanguageClick(Sender: TObject);
begin
  MnuOptLangUS.Checked := GV_Language = lng_US;
  MnuOptLangNL.Checked := GV_Language = lng_NL;
end;

procedure TMainWindow.MnuOptLangUSClick(Sender: TObject);
begin
  SetLanguage(lng_US);
end;

procedure TMainWindow.MnuOptLangNLClick(Sender: TObject);
begin
  SetLanguage(lng_NL);
end;

procedure TMainWindow.MnuHelpAboutClick(Sender: TObject);
begin
  AboutBox.ShowModal;
end;

procedure TMainWindow.FontDialogApply(Sender: TObject; Wnd: Integer);
begin
  ApplyFont;
end;

procedure TMainWindow.MnuOptSettingClick(Sender: TObject);
begin
  SettingDialog.GameSettings:= GV_GameSettings;
  if SettingDialog.ShowModal = mrOK then begin
    GV_GameSettings:= SettingDialog.GameSettings;
    AdjustPanes;
  end;
  GameReset;
  BestScore:= 0;
  AdjustStatusBar;
end;

procedure TMainWindow.TimerTimer(Sender: TObject);
begin
  Timer.Enabled:= false;
  if Playing then begin
    SndSoundOff;
    BtnStart.SetFocus;
    if GameState <> gs_Play then Playing:= false;
  end;
  ToCheckState;
end;

procedure TMainWindow.MnuGameStartClick(Sender: TObject);
begin
  ActiveGame := true;
  Melody     := '';
  ToAddNote;
end;

procedure TMainWindow.KeyClick(Sender: TObject);
var Tone: T_Tone;
begin if FindKeyTone(TButton(Sender),Tone) then begin
  if Simulating then begin
    Simulating:= false;
    ToPlayNote(Tone,Duration);
  end
  else
    ToKeyPressed(Tone);
end; end;

procedure TMainWindow.FormResize(Sender: TObject);
begin
  AdjustPanes;
end;

procedure TMainWindow.MnuHelpRulesClick(Sender: TObject);
begin ShowMessage(LS(S_Rules)); end;

end.
